import re
import typing

import click

from .scanner import run_scanner, save_data
from .tickers import download_tickers, load_tickers


def print_result(data: typing.Dict):
    click.echo(click.style(data['symbol'], fg='green', bold=True))
    click.echo(data['name'])
    click.echo(data['url'])
    click.echo(f"Mean: {data['mean']:,.0f} | StdDev: {data['std_dev']:,.0f}")
    for anomaly in data['anomalies']:
        click.echo(f"Date: {anomaly['date']} | Volume: {anomaly['volume']:,.0f} | StdDevs: {anomaly['std_devs']:,.1f}")

@click.group()
def cli():
    """
    Extract Venmo transactions from a profile with one command


    """
    pass


@cli.command()
@click.option('--months', default=5, type=click.INT, help='Months of volume history')
@click.option('--days', default=3, type=click.INT, help='Continuous days of anomalies')
@click.option('--stddev', default=5, type=click.INT, help='Standard deviation cutoff')
@click.option('--save/--no-save', default=False, help='Whether to save anomaly data')
@click.option('--filename', default='anomalies', help='Filename of which to save anomaly data')
@click.option('--filter', help='Filter on ticker symbol')
def scan(months: int, days: int, stddev: int, save: bool, filename: str, filter: str = None):
    """
    Extract Venmo transactions from a profile with one command


    """
    tickers = load_tickers()

    if filter:
        filtered_tickers = []
        compiled_filter = re.compile(filter, re.IGNORECASE)
        for ticker in tickers:
            if compiled_filter.search(f"{ticker['symbol']} - {ticker['name']}"):
                filtered_tickers.append(ticker)
        click.echo(f"Running filtered anomaly scanner on {len(filtered_tickers)} stocks")
        anomalies = run_scanner(filtered_tickers, months, days, stddev)
    else:
        click.echo(f"Running anomaly scanner on {len(tickers)} stocks")
        anomalies = run_scanner(tickers, months, days, stddev)

    for anomaly in anomalies:
        print_result(anomaly)

    if anomalies and save:
        save_data(list(anomalies), filename)


@cli.command()
@click.option('--filter', help='Filter on ticker symbol')
def list_tickers(filter: str = None):
    """
    List ticker information (saved in package data directory)
    """
    tickers = load_tickers()
    if filter:
        compiled_filter = re.compile(filter, re.IGNORECASE)
    for ticker in tickers:
        if filter:
            if compiled_filter.search(f"{ticker['symbol']} - {ticker['name']}"):
                click.echo(f"{ticker['symbol']} - {ticker['name']}")
        else:
            click.echo(f"{ticker['symbol']} - {ticker['name']}")


@cli.command()
def update_tickers():
    """
    Update ticker information (saved in package data directory)
    """
    download_tickers()
    click.echo('Successfully updated ticker information')


if __name__ == '__main__':
    cli()



import json
import os
from ftplib import FTP
import typing

# Location of package-included ticker symbols
DATA_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)), '..', 'data')
TICKER_FILE = os.path.join(DATA_DIR, 'tickers.json')


def download_tickers():
    """
    Download stock ticker symbols from NASDAQ website

    Returns:

    """
    filenames = ["otherlisted.txt", "nasdaqlisted.txt"]

    ftp = FTP("ftp.nasdaqtrader.com")
    ftp.login()
    ftp.cwd("SymbolDirectory")

    tickers = []

    for filename in filenames:
        lines = []
        ftp.retrlines(f'RETR {filename}', lines.append)

        for line in lines[1:]:
            line_split = line.strip().split("|")
            if line_split[0] == "" or line_split[1] == "" or (
                    filename == 'nasdaqlisted.txt' and line_split[6] == 'Y') or (
                    filename == 'otherlisted.txt' and line_split[4] == 'Y'):
                continue

            tickers.append({'symbol': line_split[0], 'name': line_split[1],
                            'exchange': 'other' if filename == 'otherlisted.txt' else 'nasdaq'})

        # with open(os.path.join(DATA_DIR, filename), 'wb') as fp:
        #     ftp.retrbinary(f'RETR {filename}', fp.write)

    with open(TICKER_FILE, 'w') as fw:
        json.dump(tickers, fw)


def load_tickers() -> typing.List[typing.Dict]:
    """
    Loads list of user agents
    Args:
        filepath: path to user agent file

    Returns:
        List of user agents

    """
    user_agents = []

    with open(TICKER_FILE) as f:
        tickers = json.load(f)

    return tickers


import datetime
import json
import multiprocessing
import os
import sys
import typing

import dateutil.relativedelta
import pandas as pd
import yfinance
from joblib import Parallel, delayed, parallel_backend
from tqdm import tqdm

pd.options.mode.chained_assignment = None


def download_volume_data(ticker: str, month_cutoff: int = 5) -> pd.Series:
    """

    Args:
        ticker:
        month_cutoff:

    Returns:

    """
    current_date = datetime.datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)
    cutoff_date = current_date - \
                  dateutil.relativedelta.relativedelta(months=month_cutoff)

    # Suppress Yahoo! Finance errors by re-directing system standard out
    sys.stdout = open(os.devnull, "w")

    data = yfinance.download(ticker, cutoff_date, current_date, progress=False, threads=False)

    # Replace re-directed system standard out back to normal
    sys.stdout = sys.__stdout__

    return data[["Volume"]]


def find_anomalies(volume_data: pd.Series, std_dev_cutoff: int = 10, days_cutoff: int = 3):
    """

    Args:
        volume_data:
        std_dev_cutoff:

    Returns:

    """
    volume_data.reset_index(level=0, inplace=True)
    volume_data.columns = ['date', 'volume']
    cutoff_date = volume_data["date"].max() - pd.Timedelta(days=days_cutoff)

    data_std_dev = volume_data['volume'].std()
    data_mean = volume_data['volume'].mean()
    anomaly_cut_off = data_mean + data_std_dev * std_dev_cutoff
    anomaly_data = volume_data[volume_data['volume'] > anomaly_cut_off]
    anomaly_data['std_devs'] = (volume_data['volume'] - data_mean) / data_std_dev
    anomaly_data = anomaly_data[anomaly_data['date'] >= cutoff_date]
    # print(anomaly_data['date'].dtype)
    anomaly_data['date'] = anomaly_data['date'].astype(str).str[:10]
    return anomaly_data, data_mean, data_std_dev


def parallel_wrapper(ticker: typing.Dict, months: int, days: int, stddev: int, found_anomalies: typing.List):
    volume_data = download_volume_data(ticker['symbol'], months)
    anomalies, mean, std_dev = find_anomalies(volume_data, stddev, days)

    if not anomalies.empty:
        found_anomalies.append(
            {**ticker, 'url': f"https://finance.yahoo.com/quote/{ticker['symbol']}", 'mean': mean, 'std_dev': std_dev, 'anomalies': json.loads(anomalies.to_json(orient='records'))})


def run_scanner(tickers: typing.List[typing.Dict], months: int, days: int, stddev: int):
    manager = multiprocessing.Manager()
    found_anomalies = manager.list()

    with parallel_backend('loky', n_jobs=multiprocessing.cpu_count()):
        Parallel()(delayed(parallel_wrapper)(ticker, months, days, stddev, found_anomalies)
                   for ticker in tqdm(tickers))

    return found_anomalies


def save_data(data: typing.List[typing.Dict], filename: str):
    """
    Saves scraped profile data to file
    Args:
        data:
        filename:

    Returns:

    """
    now = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
    with open(f"{filename}-{now}.json", "w") as f:
        f.write(json.dumps(data))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['unusualvolume']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'joblib>=0.16.0,<0.17.0',
 'numpy>=1.19.1,<2.0.0',
 'pandas>=1.1.0,<2.0.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'tqdm>=4.48.2,<5.0.0',
 'yfinance>=0.1.54,<0.2.0']

entry_points = \
{'console_scripts': ['unusualvolume = unusualvolume.cli:cli']}

setup_kwargs = {
    'name': 'unusualvolume',
    'version': '0.1.0',
    'description': 'Monitor stock market transactions for unusual volume and (possibly) make millions',
    'long_description': '# unusualvolume\n\nMonitor stock market transactions for unusual volume and (possibly) make millions.\n\n## Demo\n\n![](unusualvolume_demo.gif)\n\n## About\n\nA simple command line tool to retrieve stock market data (US-only) and analyze trading volumes for anomalies.\n\n## Use Cases\n\n1. Mining valuable trading information and getting ahead of possible large changes in stock prices\n2. Transaction monitoring for OSINT (open source intelligence) purposes\n3. Collecting data to train your own ML model\n\n\n## Installation\n\n```shell script\n$ pip install unusualvolume --upgrade\n```\n\n## Usage\n\n```shell script\n$ unusualvolume scan\n```',
    'author': 'Marc Ford',
    'author_email': 'mrfxyz567@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mfdeux/unusualvolume',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
